\ 2017-05-06 ew --- 
\ 2017-04-12 ew --- amforth rev 2216 or later
\                   T--Bit replaced by r11 (isrflag)
\ 32768 Hz clock crystal at timer2
\ 128 ticks/second

\ --- loaded by " make marker "
\ include lib/builds.frt
\ include lib/forth2012/core/erase.frt
\ include lib/dot-base.frt
\ include lib-avr8/imove.frt
\ include lib-avr8/bitnames.frt
\ include lib-avr8/forth2012/core-ext/marker.frt
\ include lib-avr8/forth2012/core/environment-q.frt
\ include lib-avr8/dot-res.frt
\ include lib-avr8/forth2012/core/avr-values.frt
\ include lib/forth2012/core/is.frt
\ include lib/forth2012/tools/dumper.frt
\ include atmega644p.fs
\ include first.fs

marker --start--

$0061 Evalue stationID
$0012 Evalue swVersion

\ --- ports, pins, masks

PORTA 1 portpin: _hd44780-rs
PORTA 2 portpin: _hd44780-rw
PORTA 3 portpin: _hd44780-en
$f0    constant hd44780-data-mask
PORTA  constant hd44780-port
DDRA   constant hd44780-ddr

PORTB 0 portpin: lederr
PORTB 1 portpin: ledsec
PORTB 2 portpin: ledsensor 
PORTB 3 portpin: leddata

\ abakus display
PORTD 2 portpin: sr_latch
PORTD 3 portpin: sr_clock
PORTD 4 portpin: sr_data

\ dump irq counts
\ fixme: #irqs not defined?
: irqdump #31 0 do 
    i irq[]#    #4 .r space  \ addr
    i 1+        #2 .r space  \ int number
    i irq[]# c@    u. cr     \ count mod $FF
  loop ;

\ --- famous includes and other words
: ms        ( n -- )     0 ?do pause 1ms loop ;
: u0.r      ( u n -- )   >r 0 <# r> 0 ?do # loop #> type ;

\ make redirects return to console
' emit defer@ constant emit.orig
: /lcd  emit.orig to emit ;

include ewlib/flags.fs          \ flag: fset fclr fset? fclr?
include lib/forth2012/double/2variable.frt
include lib/forth2012/double/2constant.frt
include lib/forth2012/double/2-fetch.frt
include lib/forth2012/double/2-store.frt
include lib/forth2012/double/m-star-slash.frt
include lib/quotations.frt

\ --- stationID prompt
\     .ready is a deferred word since AmForth 6.3
: .stationID_ready
  cr
  [char] ~ emit
  base @  $10 base !  stationID 2 u0.r  base !
  [char] > emit space
;

\ --- driver: status leds
include ewlib/leds.fs

\ --- driver: 2x8 cheap pollin LCD (4bit mode)
include ewlib/hd44780-4bit.fs
include ewlib/lcd_2x8_pollin.fs 
                                \ lcd.pos ( row col -- )
                                \ >lcd  /lcd  redirect
                                \ lcd.page ( -- )
: lcd-msg
  hex lcd.page >lcd
  0 0 lcd.pos   ." stn " stationID 4 u0.r
  1 0 lcd.pos   ." ver " swVersion 4 u0.r
  /lcd
;

include ewlib/shiftregister.fs  \ +sr >sr
include ewlib/abakus.fs         \ emit.abakus
                                \ type.abakus


\ --- driver: i2c rtc clock
include ewlib/twi.fs
include ewlib/i2c.fs
include ewlib/i2c_rtc.fs \ needs year ,, second

\ --- timeup
include ewlib/clocks_v0.1.fs    \ newfangled clock data as array
                                \ clock:
                                \ sec min hour day month year
                                \ .date .time .tz
s" UT" clock: UT                \ create the "master" clock

\ include ewlib/timeup_2.2.fs     \ timeup: time/date system with jobs
include ewlib/timeup_3.0.fs     \ timeup: time/date system with jobs
                                \ tick sec min hour day month year
                                \ leap_year? length_of_month
                                \ timeup.init
                                \ tu.set  ( Y m d H M S -- )
                                \ tu.get  ( -- S M H d m Y )
                                \ tu.show ( -- )

: hwclock>timeup ( -- )
  rtc.get    \ --
             year  !
  bcd>dec 1- month !
  bcd>dec 1- day   !
  bcd>dec    hour  !
  bcd>dec    min   !
  bcd>dec    sec   !
  drop \ 1/100 secs
  year @   month @ 1+  tu.upd.limits
;
: timeup>hwclock ( -- )

  year @   month @ 1+  day @ 1+
  hour @   min   @     sec @
  0 \ tick @ #100 #128 m*/
  ( Y m d H M S S/100 ) rtc.set
;

\ --- clock display (deferred)
include lib-avr8/forth2012/core-ext/avr-defers.frt
include ewlib/clock_display_defer.fs
include ewlib/clock_display_pollin_2x8.fs
                                \ +clock.display.pollin_2x8
\ --- timer2 clock tick
include lib-avr8/eallot.frt
include lib-avr8/2evalue.frt
2variable uptime2
: .uptime2  ( -- ) uptime2 2@ decimal ud. [char] s emit ;
: ++uptime2 ( -- ) 1.  uptime2 2@  d+  uptime2 2! ;

include ewlib/clock_tick_v5.0.fs


\ --- epoch seconds, timezones
: u>= ( n n -- t/f ) u< invert ;
: d>s ( d -- n ) drop ;
#2015        Evalue   EE_last_epoch
           variable   _last_epoch
1420070400. 2Evalue   EE_last_esec
          2variable   _last_esec
include ewlib/epochseconds.fs
2variable Esec
: ++Esec  ( -- )    Esec 2@  1. d+  Esec 2! ;
: .Esec   ( -- )    Esec 2@ ud. ;
2variable EsecOffset
: GMT  ( -- )     0. EsecOffset 2! ;
: MEZ  ( -- )  3600. EsecOffset 2! ;
: MESZ ( -- )  7200. EsecOffset 2! ;
: dot.time  #2 u0.r [char] : emit  #2 u0.r [char] : emit #2 u0.r ;
: dot.date  #4 u0.r  #2 u0.r  #2 u0.r ;

: cd.line0.S
  decimal >lcd 0 6 lcd.pos  sec @ #2 u0.r /lcd
;
: cd.line0.time
  decimal >lcd
  0 0 lcd.pos 
  Esec 2@  EsecOffset  2@ d+  s>dt.short drop drop drop dot.time
  /lcd
;  

\ --- multitasker
include lib/multitask.frt       \ activate ( tid -- )
                                \ task-sleep ( tid -- )
                                \ task-awake ( tid -- )
                                \ task ( rs-size ds-size -- tid )
                                \ alsotask ( tid -- )
                                \ single ( -- )
                                \ multi ( -- )
                                \ onlytask ( -- )
                                \ tasks ( -- )
: +tasks  multi ;
: -tasks  single ;


\ --- timeup jobs ------------------------------------------
: job.tick  
  leddata toggle
;
: job.sec
  ledsec toggle
  ++uptime2
  ++Esec
  cd.time.S
  cd.line0.S

;
: job.min
  lederr toggle
  \ cd.date
  cd.line0.time
  cd.time
;
: job.hour  ;
: job.day   ;
: job.month
  \ update length of month in tu.limits
  year @  month @ 1+  tu.upd.limits
  ;
: job.year
  \ update YYYY in eeprom of rtc
  year @  rtc.set.year
;

create Jobs
  ' job.tick ,
  ' job.sec , ' job.min ,   ' job.hour ,
  ' job.day , ' job.month , ' job.year ,

variable jobCount
: jobCount++
  jobCount @
  6 < if
    1 jobCount +!
  then
;

\ --- task 2 -----------------------------------------------
: msBit?  ( x -- t/f ) $8000 and 0= 0= ;

variable last.msBit
: run-job
  ['] tx-poll to emit \ add emit to inner workings of run-job
  begin
    ledsensor toggle

    tick.over? if
      1 ct.ticks2.follow +!

      \ advance phase:
      ct.phase 2@   ct.fcontrol 2@ d+ ct.phase 2!

      UT  1 tick +!
      job.tick
    then



    \ check msBit for "falling edge" == second over
    ct.phase 2@ swap ( LOW ) drop 
    ( HIGH ) msBit?
    dup last.msBit @   =  if
      drop
    else
      \ half second over
      ( msbit ) dup 0= if
        \ second over
        UT timeup   0 tick !
        1 jobCount !
      then
      ( msBit ) last.msBit  !
    then
    
    \ these are run one job per loop, not all in one go.
    jobCount @
    bv tu.flags fset?
    if
      jobCount @ dup
      Jobs + @i execute
      bv tu.flags fclr
    then
    jobCount++

    pause
  again
;
$40 $40 0 task: task_job    \ create task space
: start-job
  task_job tib>tcb
  activate \ words after this line are run in new task
  run-job
;
: starttasker
  task_job task-init        \ create TCB in RAM
  start-job                 \ activate tasks job

  onlytask                  \ make cmd loop task-1
  task_job tib>tcb alsotask \ start task-2
  multi                     \ activate multitasking
;


\ --- main -------------------------------------------------
: init
  
  hd44780-4bit-init lcd.page
  +leds leds-intro
  lcd-msg #2000 ms
  ['] .stationID_ready is .ready
  +twi

  UT MESZ 
  0. uptime2 2!
  0. Esec    2!
  EE_last_epoch _last_epoch  !
  EE_last_esec  _last_esec  2!
  +ticks2
  timeup.init
  0 last.msBit  !
  
  i2c_addr_rtc twi.ping? if
    hwclock>timeup
  else
    #2015 1 1 0 0 0 tu.set
  then
  tu.get ut>s.short Esec 2!

  -clock.display
  \ LCD
  +clock.display.pollin_2x8
  lcd.page
  cd.line0.time
  cd.time
;

: run
  init
  starttasker
;

: run-turnkey
  applturnkey
  init
  starttasker
;
\ ' run-turnkey to turnkey


: .dt ( year month day hour min sec -- )
  #4 u0.r #2 u0.r #2 u0.r
  [char] - emit
  #2 u0.r [char] : emit  #2 u0.r [char] : emit  #2 u0.r
;

: .d ( -- )
  decimal
  .uptime2 space space
  tu.show space 
  tick @ . space space
  ct.ticks2.follow @ . space space
  .Esec space
  Esec 2@  EsecOffset  2@ d+  s>dt.short .dt
  cr
;
